/*!
 * JS objects used for the Pacelli Website
 *
 * Dependencies: jQuery, SlimWidgets
 *
 */

(function ($, Slim) {

    var objs = {}; 
    
    // CSS class names
    objs.classNames = {

        singleViewBtn: {
            main: 'btn btn-sm float-right single-view-btn',
            icon: 'fas fa-file-import'
        },
        singleViewIcon: 'fas fa-arrow-left move-left-btn',
        moveToTopIcon: 'fas fa-arrow-up move-to-top-btn',
        closeIcon: 'fas fa-times close-btn',
        // CSS class names for LinkItem objects     
        LinkItem: {
            icon: 'toggle-resource-btn',
            iconOpen: 'fas fa-chevron-up',
            iconClosed: 'fas fa-chevron-down'
        },
        // CSS class names for AccoTitle objects
        AccoTitle: {
            iconOpen: 'fas fa-chevron-up',
            iconClosed: 'fas fa-chevron-down',
            iconDisabled: 'fas fa-ban'
        }
    };
    
    // CSS class selectors
    objs.selectors = {
        accordion: '.accordion',
        accoTitleIcon: '.fas',
        bioLink: '.bio-link',
        bioLinkItem: '.bio-link-item',
        bioLinkIcon: '.bio-link-icon',
        resourceTypeControlBtn: '.resource-type-control-btn',
        docLinkItem: '.doc-link-item',
        docLinkItemsGroup: '.doc-link-items-group',
        docTypeControlBtn: '.doc-type-control-btn',
        findbuchBtn: '.gnd-btn',
        footnote: '.footnote',
        footnoteReverse: '.footnote-reverse',
        linkItem: '.link-item',
        linkItems: '.link-items',
        moveToTopIcon: '.move-to-top-btn', 
        pdfBtn: '.pdf-btn',
        subLink: '.sub-link',
        subLinkIcon: '.sub-link-icon',    
        subLinkItem: '.sub-link-item'
    };
    
    objs.ajax = {}; 
    
    objs.ajax.urls = {
        // Get a single resource
        getBio: './resources/html/ajax/get-bio.html',
        getSub: './resources/html/ajax/get-sub.html',
        getDocument: './resources/html/ajax/get-document.html',
        // Get documents associated with a single bio
        docsToBio: './resources/html/ajax/get-docs-linking-to-bio.html',
        // Get documents associated with a single sub
        docsToSub: './resources/html/ajax/get-docs-linking-to-sub.html',
        // Get documents, bios and subs associated with a single document
        attachInDoc: './resources/html/ajax/get-linked-attachments.html',
        biosInDoc: './resources/html/ajax/get-linked-bios.html',
        docsInDoc: './resources/html/ajax/get-linked-docs.html',
        docsToDoc: './resources/html/ajax/get-docs-linking-to-doc.html',
        entwAusfInDoc: './resources/html/ajax/get-linked-entwAusfs.html',
        subsInDoc: './resources/html/ajax/get-linked-subs.html',
        weisungInDoc: './resources/html/ajax/get-linked-weisungen.html'
    }
    
    
    /* Utility functions/shared functions
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */ 
    
    objs.util = {}; 
    
    /* Create a new object with the given prototype
     * See: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object/create)
     */
    objs.util.createObject = Object.create || function(proto){
        function ctor() {}
        ctor.prototype = proto;
        return new ctor();
    };
    
    /* Load custom content via ajax
     *
     * @param url url for the ajax request (see http://api.jquery.com/jquery.ajax/) 
     * @param data data (parameters) for the ajax request
     * @param initFn success function for the ajax request
     * @param context context for function "initFn"
     * @param arguments an array of custom additional parameters for function "initFn"
     */
    objs.util.loadContent = function(url, data, initFn, context, arguments){
        
        var loadingSpinner = Slim.LoadingSpinner().append();
        var args = arguments instanceof Array ? arguments : [];
        
        $.ajax({
            url: url,
            data: data
        }).done(function(data){
            
            args.unshift(data); // provide the loaded data as first paramter for function initFn
            initFn.apply(context, args); 
            
        }).always(function(){
            loadingSpinner.remove();
        });  
    };
    
    // Like "objs.util.loadContent" but without loading spinner
    objs.util.loadContentNoLs = function(url, data, initFn, context, arguments){
        
        var args = arguments instanceof Array ? arguments : [];
        
        $.ajax({
            url: url,
            data: data
        }).done(function(data){
            
            args.unshift(data); // provide the loaded data as first paramter for function initFn
            initFn.apply(context, args); 
        });  
    };
    
    // Determine the url required for loading a single resource via ajax
    objs.util.getResourceUrl = function(type){
    
        if (type === 'bio'){
            return objs.ajax.urls.getBio;
        } else if (type === 'sub'){
            return objs.ajax.urls.getSub;
        } else if (type === 'doc'){
            return objs.ajax.urls.getDocument;
        }
    }; 
    
    // Create a single resource
    objs.util.createResource = function(resourceData, type, extensions){ 

        if (type === 'bio'){
            return new objs.Bio(resourceData, extensions);
        } else if (type === 'sub'){
            return new objs.Sub(resourceData, extensions);  
        } else if (type === 'doc'){
            return new objs.Doc(resourceData, extensions);  
        }
    };
    
    // Determine the type of the current resource
    objs.util.getResourceType = function(resource){
        
        if (resource instanceof objs.Bio){
            return 'bio';
        } else if (resource instanceof objs.Sub){
            return 'sub';
        } else if (resource instanceof objs.Doc){
            return 'doc';
        }
    };  
    
    // Create a matching ResultSet for the current accoTitle-type
    objs.util.createResultSet = function(data, type, resourceExtensions){
    
        if (type === 'biosInDoc'){
            return new objs.ResultSet(data, 'bio', resourceExtensions); 
        } else if (type === 'subsInDoc') {
            return new objs.ResultSet(data, 'sub', resourceExtensions);   
        } else {
            return new objs.ResultSet(data, 'doc', resourceExtensions);     
        }
    };
    
    
    /* More functions
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */ 
    
    objs.more = {};

    /* Load further web resources for a single bio or sub using its GND and the seeAlso service under https://data.deutsche-biographie.de/beta/beacon-open/
     * Display the resulting links as popup 
     */
    objs.more.initFindbuchBtn = function($btn){
        var ajax = {
            options: {
                url: './resources/html/ajax/get-beacon-service-resources.html',  
                data: {
                    targetUrlPrefix:'http://data.deutsche-biographie.de/rest/bd/gnd/',
                    targetUrlSuffix: '/alle_de'
                }, 
                cache: false,
                crossDomain: true,
                dataType: 'XML',
                timeout: 5000
            },
            // Handle findbuch data (success function for ajax call in Slim.bindAjaxPopupTo)  
            callback: function(data, popupOptions){ 
                var $container = $(document.createElement('div')).addClass('findbuch-resources'); 
                if (location.pathname.includes("/en/")) {
                    var popupTitle = 'Further web resources';
                    var noWebResources = 'No other web resources could be identified for this short biography.';
                    var findbuchInfo = '<p>The links above were automatically retrieved using the GND number '
                        + 'via the SeeAlso service of the site <a target="_blank" href="https://data.deutsche-biographie.de/beta/beacon-open/">https://data.deutsche-biographie.de/beta/beacon-open/</a></p>';
                } else {
                    var popupTitle = 'Weitere Webressourcen';
                    var noWebResources = 'Zu dieser Kurzbiografie konnten keine weiteren Webressourcen ermittelt werden.';
                    var findbuchInfo = '<p>Oben stehende Links wurden unter Nutzung der GND-Nummer automatisch ermittelt '
                        + 'über den SeeAlso-Dienst der Seite <a target="_blank" href="https://data.deutsche-biographie.de/beta/beacon-open/">https://data.deutsche-biographie.de/beta/beacon-open/</a></p>';
                }
                
                $container.append(new XMLSerializer().serializeToString(data))
                $container.append(findbuchInfo);
                
                /*if (!data[0] && data[1].length === 0){  // No data
                     $(document.createElement('p')).text(noWebResources)
                        .appendTo($container);
                
                } else {
                    
                    // Special treatment for the first item ('Deutsche Nationalbibliothek')
                    $(document.createElement('a'))
                        .html('Deutsche Nationalbibliothek')
                        .attr('href', data[0])
                        .attr('target', '_blank')
                        .appendTo($container);
                
                    // Handling all other items
                    for (var i=0; i < data[1].length; i++){
                        $(document.createElement('a'))
                            .html(data[1][i])
                            .attr('href', data[3][i])
                            .attr('target', '_blank')
                            .appendTo($container);
                    }
                    
                    
                    $container.append(findbuchInfo);
                }*/
                
                return Slim.Popup({title: popupTitle, content: $container}, popupOptions); 
            }
        }
        
        Slim.bindAjaxPopupTo($btn, ajax);
    };
    

    /* Content 
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */    
    
    /*
     * Content: parent class Resource and ResultSet
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current Content instance
     */
    objs.Content = function(selector){
        this.$self = $(selector);
    };
    
    // Show the (jQuery wrapped) DOM node associated with the current Content object
    objs.Content.prototype.show = function(){
        this.$self.show();
    }; 
    
    // Hide the (jQuery wrapped) DOM node associated with the current Content object
    objs.Content.prototype.hide = function(){
        this.$self.hide();
    }; 
    
    // Insert the (jQuery wrapped) DOM node associated with the current Content object after a given node (target)
    objs.Content.prototype.insertAfter = function(target){
        this.$self.insertAfter(target);
    }; 
    
    // Insert the (jQuery wrapped) DOM node associated with the current Content object before a given node (target)
    objs.Content.prototype.insertBefore = function(target){
        this.$self.insertBefore(target);
    }; 
    
    // Remove the (jQuery wrapped) DOM node associated with the current Content object
    objs.Content.prototype.remove = function(){
        this.$self.remove();
    }; 


    /* Resource -> Content
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    /* Resource: Parent class for Bio, Sub and Doc
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current Resource instance
     * @param extensions an object specifing the type of extension(s) for the current Resource instance, e.g. {closeBtn:true} -> Add close button
     */
    objs.Resource = function(selector, extensions){
    
        objs.Content.call(this, selector);
        
        var $bioAndSubLinks = this.$self.find(objs.selectors.bioLink + ',' + objs.selectors.subLink);
        var $bioLinkIcons = this.$self.find(objs.selectors.bioLinkIcon);
        var $subLinkIcons = this.$self.find(objs.selectors.subLinkIcon);
        
        function popupBio(data){
            Slim.Popup({content: new objs.Bio(data).$self}).append();
        }
        
        function popupSub(data){
            Slim.Popup({content: new objs.Sub(data).$self}).append();
        }
        
        // Direct links to bios and subs  no longer needed
        $bioAndSubLinks.removeAttr('href');  
        
        // Click icon to show linked bio as popup
        $bioLinkIcons.click(function(){
            objs.util.loadContent(objs.ajax.urls.getBio, {idno: $(this).attr('data-idno')}, popupBio);
        });
        
        // Click icon to show linked sub as popup
        $subLinkIcons.click(function(){
            objs.util.loadContent(objs.ajax.urls.getSub, {idno: $(this).attr('data-idno')}, popupSub);
        });        
        
        if (extensions){
            
            if (extensions.singleViewBtn){
                
                this.singleViewBtn; 
                this.addSingleViewBtn(); 
            } 
            
            if (extensions.singleViewIcon) {
                
                this.singleViewIcon; 
                this.addSingleViewIcon();   
            } 
            
            if (extensions.moveToTopIcon){
                
                this.moveToTopIcon; 
                this.addMoveToTopIcon();   
            } 
            
            if (extensions.closeIcon){
                
                this.closeIcon; 
                this.addCloseIcon();   
            }
        }
    };
    
    // Inherit prototype from class Content
    objs.Resource.prototype = objs.util.createObject(objs.Content.prototype);
    objs.Resource.prototype.constructor = objs.Resource;  
    
    // Get the url for the single view of the current resource
    objs.Resource.prototype.getSingleViewUrl = function(){
        
        return this.$self.find(objs.selectors.pdfBtn).attr('href').replace(/-pdf/, '');
    };
    
    // Add button that allows to open the resource in an own window
    objs.Resource.prototype.addSingleViewBtn = function(){ 
        
        if (this.singleViewBtn) { return false; }
        
        this.singleViewBtn = $(document.createElement('a'))
            .addClass(objs.classNames.singleViewBtn.main)
            .attr('href', this.getSingleViewUrl())
            .text('Einzelansicht')
            .append('<i class="' + objs.classNames.singleViewBtn.icon + '"></i>')
            .insertAfter(this.$self.find(objs.selectors.pdfBtn));
    };
    
    // Add icon that allows to open the resource in an own window
    objs.Resource.prototype.addSingleViewIcon = function(){ 
        
        if (this.singleViewIcon) { return false; }
        
        this.singleViewIcon = $(document.createElement('i'))
            .addClass(objs.classNames.singleViewIcon)
            .attr('title', 'Nach links verschieben/Als Hauptdokument anzeigen')
            .appendTo(this.$self)
            .click(function(){ 
                location.href = this.getSingleViewUrl();
            }.bind(this));
    };
    
    // Add icon that allows to move a copy of the resource to top (top = objs.selectors.accordion) 
    objs.Resource.prototype.addMoveToTopIcon = function(){
        
        if (this.moveToTopIcon) { return false; }
        
        this.moveToTopIcon = $(document.createElement('i'))
            .addClass(objs.classNames.moveToTopIcon)
            .attr('title', 'Nach oben verschieben/Parallel zum Hauptdokument anzeigen')
            .appendTo(this.$self)
            .click(function(){ 
            
                var resourceData = this.$self.clone(); 
                var type = objs.util.getResourceType(this);
                var resource = objs.util.createResource(resourceData, type, {closeIcon:true, singleViewIcon:true}); 
                
                resource.$self.find(objs.selectors.moveToTopIcon).remove(); 
                resource.insertBefore(objs.selectors.accordion); 
                
                $('html, body').animate({scrollTop: 220}, 500);
                
            }.bind(this)); 
    }; 
    
    // Add icon that allows to close/remove the resource
    objs.Resource.prototype.addCloseIcon = function(){ 
        
        if (this.closeIcon) { return false; }
        
        this.closeIcon = $(document.createElement('i'))
            .addClass(objs.classNames.closeIcon)
            .attr('title', 'Schließen')
            .appendTo(this.$self)
            .click(function(){ 
                this.remove();
            }.bind(this));
    };
    
    
    /* Bio -> Resource
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    /* Bio: biography (Kurzbiografie)
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current Bio instance
     * @param extensions an object specifing the type of extension(s) for the current Bio instance, e.g. {closeBtn:true} -> Add close button
     */
    objs.Bio = function(selector, extensions){
        
        objs.Resource.call(this, selector, extensions);
        
        var $findbuchBtn = this.$self.find(objs.selectors.findbuchBtn);
        
        if ($findbuchBtn){
            objs.more.initFindbuchBtn($findbuchBtn); 
        }
    };
    
    // Inherit prototype from class Resource
    objs.Bio.prototype = objs.util.createObject(objs.Resource.prototype);
    objs.Bio.prototype.constructor = objs.Bio;  
    
    
    /* Sub -> Bio
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    /* Sub: subject (Schlagwort)
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current Sub instance
     * @param extensions an object specifing the type of extension(s) for the current Sub instance, e.g. {closeBtn:true} -> Add close button
     */
    objs.Sub = function(selector, extensions){
        
        objs.Bio.call(this, selector, extensions);
    };
    
    // Inherit prototype from class Bio
    objs.Sub.prototype = objs.util.createObject(objs.Bio.prototype);
    objs.Sub.prototype.constructor = objs.Sub;  
    
    
    /* Doc -> Resource
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    /* Doc: document (Dokument)
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current Doc instance
     * @param extensions an object specifing the type of extension(s) for the current Doc instance, e.g. {closeBtn:true} -> Add close button
     * 
     * Note: Layermodel is not handled here but within pacelli.js, function 'PA.pages.doc'
     */
    objs.Doc = function(selector, extensions){
        
        objs.Resource.call(this, selector, extensions);
        
        // Transform footnote anchors
        $(objs.selectors.footnote + ',' + objs.selectors.footnoteReverse).click(function(e){
            
            e.preventDefault(); 
            e.stopPropagation();
            
            var id = $(this).attr('href');
            
            console.log(id);
            console.log($(id))
            
            $('html, body').animate({
                scrollTop: $(id).offset().top
            }, 500);
        });
        

        // ....
    };
    
    // Inherit prototype from class Resource
    objs.Doc.prototype = objs.util.createObject(objs.Resource.prototype);
    objs.Doc.prototype.constructor = objs.Doc;  
   
   
    /* ResultSet -> Content
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */    
    
    /* ResultSet: A set of LinkItems
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current ResultSet instance
     * @param type specifies the type of LinkItems and asscociated Resources contained in the current ResultSet
     * @param extensions an object specifing the type of extension(s) for the Resource objects contained within the current ResultSet instance, 
     *  e.g. {closeBtn:true} -> Add close button
     */
    objs.ResultSet = function(selector, type, resourceExtensions){ 
        
        objs.Content.call(this, selector);      
        
        this.$self.find(objs.selectors.bioLinkItem).each(function(){
            new objs.LinkItem(this, 'bio', $(this).find('>a').attr('data-idno'), resourceExtensions)
        });
        
        this.$self.find(objs.selectors.subLinkItem).each(function(){
            new objs.LinkItem(this, 'sub', $(this).find('>a').attr('data-idno'), resourceExtensions)
        });
        
        this.$self.find(objs.selectors.docLinkItem).each(function(){
            new objs.LinkItem(this, 'doc', $(this).find('>a').attr('data-idno'), resourceExtensions)
        });
        
        if (type === 'doc'){
        
            this.initDocTypeNav(); 
        }
        
        if (type === 'mixed'){
            
            this.initDocTypeNav(); 
            this.addResouceTypeNav(); 
        } 
    };
       
    // Inherit prototype from class Resource
    objs.ResultSet.prototype = objs.util.createObject(objs.Content.prototype);
    objs.ResultSet.prototype.constructor = objs.ResultSet;  
    
    // Switch between different doc classes (Bericht Ausf., Bericht Entw., Anlage, Weisung)
    objs.ResultSet.prototype.initDocTypeNav = function(){
        
        var $ctrlBtns = this.$self.find(objs.selectors.docTypeControlBtn); 
        
        // Show all LinkItems having the type of the currently clicked radio button while hiding all others 
        function switchDocClass(){

            var $btn = $(this);
            var $docLinkItemsGroups = $btn.closest(objs.selectors.linkItems).find(objs.selectors.docLinkItemsGroup); 
            var value = $btn.attr('value');
            
            $docLinkItemsGroups.hide();
            
            if (value === 'alle'){
                $docLinkItemsGroups.show();
            } else {
                $docLinkItemsGroups.filter('[data-classtype='+value+']').show();
            }
            
            $ctrlBtns.removeClass('active');
            $btn.addClass('active');
        }
        
        // Activate or disable a single radio button depending on the number of associated LinkItems
        function initDocTypeNavBtn($btn){
            

            var value = $btn.attr('value'); 
            var $linkItems = $btn.closest(objs.selectors.linkItems).find(objs.selectors.linkItem); 
            var nmbItems = (value === 'alle') ? $linkItems.length : $linkItems.filter('[data-classtype='+value+']').length;

            $btn.append(' ' + nmbItems) 

            if (nmbItems === 0){
                $btn.addClass('disabled');
                $btn.removeAttr('title');
            } else {
                $btn.click(switchDocClass); 
            }
        };
        
        $ctrlBtns.each(function(){
           initDocTypeNavBtn($(this));
        }); 

        $ctrlBtns.not('.disabled').first().click();
    };
    
    // Switch between result sets for different resource types (Bio-, Sub-, Doc-LinkItems) 
    objs.ResultSet.prototype.addResouceTypeNav = function(){
        
        var $ctrlBtns = this.$self.find(objs.selectors.resourceTypeControlBtn);
        
        function switchResouceType(e){
            
            var $btn = $(e.target);
            var $itemsToShow = this.$self.find('.' + $btn.attr('data-target')); 
            
            if ($itemsToShow.length === 0){ return false; }
            
            $btn.siblings().removeClass('active');
            $btn.addClass('active');
            
            $itemsToShow.siblings(objs.selectors.linkItems).hide(); 
            $itemsToShow.show();  
        }; 
        
        $ctrlBtns.click(switchResouceType.bind(this));
        $ctrlBtns.first().click();
    };
    

    /* ToggleContentItem
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */    
   
    /*
     * ToggleContentItem: Parent class for LinkItem and AccoTitle
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current ToggleContentItem instance
     * @param type to be specified in child classes
     * @param idno identifier for the asscociated resource on the server
     */
    objs.ToggleContentItem = function(selector, type, idno){
        
        this.$self = $(selector);
        this.$icon = $();
        
        this.type = type; 
        this.idno = idno; 
        this.url; 
        this.state = 'closed';
        this.classNames = {
            open: 'open', 
            iconOpen: '',
            iconClosed: ''
        }
        this.content;
        this.initContentArgs; 
    }
    
    // Open/close ToggleContentItem, show/hide its content
    objs.ToggleContentItem.prototype.toggle = function(){ 
    
        // Ajax request pending
        if (this.state === 'blocked'){ return false }; 
        
        if (this.state === 'closed'){   // Item currently closed
            
            // Content already loaded -> show
            if (this.content){
                
                this.open(); 
            
            } else {    // Load content via ajax
            
                var data = this.getAjaxData();
            
                this.state = 'blocked';
                objs.util.loadContent(this.url, data, this.initContent, this, this.initContentArgs); 
            }
            
        } else {    // Item currently open
        
            this.close();   // Hide content
        } 
    };
    
    // Provide data for an ajax request
    objs.ToggleContentItem.prototype.getAjaxData = function(){
        return {idno:this.idno}; 
    };
    
    /* Initialize content loaded from the server. Used as succes function within an ajax request. Expected to be overriden in child classes.
     * Any number of arguments (to be defined as array in property "this.initContentArgs") can be used after 'data', e.g.
     * this.initContentArgs = [arg1, arg2, arg3], ...initContent = function(data, arg1, arg2, arg3){...}
     */
    objs.ToggleContentItem.prototype.initContent = function(data){};
    
    // Open ToggleContentItem, show its content
    objs.ToggleContentItem.prototype.open = function(){
        
        this.$self.addClass(this.classNames.open);
        this.$icon.removeClass(this.classNames.iconClosed);
        this.$icon.addClass(this.classNames.iconOpen);
        
        this.state = 'open'; 
        
        if (this.content){
            this.content.show(); 
        }
    };
    
    // Close ToggleContentItem, hide its content
    objs.ToggleContentItem.prototype.close = function(){
        
        this.$self.removeClass(this.classNames.open);
        this.$icon.removeClass(this.classNames.iconOpen);
        this.$icon.addClass(this.classNames.iconClosed);
        
        this.state = 'closed'; 
        
        if (this.content){
            this.content.hide(); 
        }
    };
    
    
    /* LinkItem -> ToggleContentItem
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */  
    
    /*
     * LinkItem: an item providing acces to a Bio, Sub or Doc
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current LinkItem instance
     * @param type 'bio', 'sub', or 'doc'
     * @param idno identifier for the asscociated resource on the server
     * @param extensions an object specifing the type of extension(s) for the Resource objects associated within the current LinkItem instance, 
     *  e.g. {closeBtn:true} -> Add close button
     */
    objs.LinkItem = function(selector, type, idno, resourceExtensions){
    
        objs.ToggleContentItem.call(this, selector, type, idno);
        
        // Optional extensions for the resources handled by class LinkItem
        this.resourceExtensions = resourceExtensions;
        
        // Set url
        this.url = objs.util.getResourceUrl(type); 
        
        // Set class names
        this.classNames.iconOpen = objs.classNames.LinkItem.iconOpen;
        this.classNames.iconClosed = objs.classNames.LinkItem.iconClosed;
        
        // Create ineractive icon
        this.$icon = $(document.createElement('i'))
            .addClass(objs.classNames.LinkItem.icon + ' ' + this.classNames.iconClosed)
            .click(this.toggle.bind(this))
            .appendTo(this.$self);
    };
    
    // Inherit prototype from ToggleContentItem
    objs.LinkItem.prototype = objs.util.createObject(objs.ToggleContentItem.prototype);
    objs.LinkItem.prototype.constructor = objs.LinkItem;  
    
    // Append a single Resource object after the current LinkItem
    objs.LinkItem.prototype.initContent = function(resourceData){

        this.content = objs.util.createResource(resourceData, this.type, this.resourceExtensions); // Init Resource
        this.content.insertAfter(this.$self);   // Append Resource
        this.open(); // Open LinkItem
    };
    
    
    /* AccoTitle -> ToggleContentItem
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */    
    
    /*
     * AccoTitle: Provides access to a result set
     * 
     * @param selector (selector for) the (jQuery wrapped) DOM node associated with the current LinkItem instance
     * @param type specifies the type of the associated result set
     * @param idno identifier for the asscociated resource on the server
     * @param extensions an object specifing the type of extension(s) for the Resource objects associated within the current ResultSet instance, 
     *  e.g. {closeBtn:true} -> Add close button
     */
    objs.AccoTitle = function(selector, type, idno, resourceExtensions, autoOpen){
        
        objs.ToggleContentItem.call(this, selector, type, idno);
        
        // Optional extensions for the resources handled by class LinkItem
        this.resourceExtensions = resourceExtensions;
        
        // Set url
        this.url = objs.ajax.urls[type]; 

        // Set class names
        this.classNames.iconOpen = objs.classNames.AccoTitle.iconOpen;
        this.classNames.iconClosed = objs.classNames.AccoTitle.iconClosed;
        
        this.$icon = this.$self.find(objs.selectors.accoTitleIcon);
        
        this.autoOpen = autoOpen; 
        
        this.$self.removeAttr('href');  // Direct link to result set no longer needed
        
        this.checkForContent(); 
    }
    
    // Inherit prototype from ToggleContentItem
    objs.AccoTitle.prototype = objs.util.createObject(objs.ToggleContentItem.prototype);
    objs.AccoTitle.prototype.constructor = objs.AccoTitle;  
    
    // Check if there is associated content for the current AccoTitle instance on the server and deactivate it otherwise
    objs.AccoTitle.prototype.checkForContent = function(){

        // Request associated content from the server
        function evaluate(resultData){  

            var clearResult = $(resultData).text().trim();
            var nmbResults = parseInt(clearResult); 
            
            // this.$self.append(' ' + clearResult); 
            this.$self.prepend('<span style="display:inline-block; width:50px;">' + clearResult + '</span>'); 
            
            if (nmbResults > 0){
                
                this.$self.click(this.toggle.bind(this));
                
                if (this.autoOpen){    // Open AccoTitle directly
                    
                    objs.util.loadContentNoLs(this.url, this.getAjaxData(), this.initContent, this, this.initContentArgs); // Will change state to "open"
                    
                } else {     
                
                    this.state = 'closed'; // Default state
                }
                
            } else {    // No associated content
                
                this.$self.addClass('disabled');
                this.$icon.removeAttr('class')
                    .addClass(objs.classNames.AccoTitle.iconDisabled); 
            }
        };
        
        this.state = 'blocked';
        
        objs.util.loadContentNoLs(this.url, this.getAjaxData('true'), evaluate, this, this.initContentArgs);
    }; 
    
    // Append a single Resource object after the current LinkItem
    objs.AccoTitle.prototype.initContent = function(resourceData){

        this.content = objs.util.createResultSet(resourceData, this.type, this.resourceExtensions); // Init ResultSet
        this.content.insertAfter(this.$self);   // Append ResultSet
        this.open(); // Open LinkItem
    };
    
    // Provide data for an ajax request, overrides parent function
    objs.AccoTitle.prototype.getAjaxData= function(countOnly){
        
        var data = {
            idno:this.idno, 
            countOnly: countOnly
        };
        
        data[this.type] = 'show';
        
        return data
    };
    
    window.PAO = objs; 
    
}(jQuery, Slim));